<?php

include_once(_PS_MODULE_DIR_ . 'imoje/libraries/Helper.php');

if(!defined('_PS_VERSION_')) {
	exit;
}

/**
 * Class Imoje
 *
 * @property string name
 * @property string tab
 * @property string version
 * @property string author
 * @property int    need_instance
 * @property bool   currencies
 * @property string currencies_mode
 * @property array  ps_versions_compliancy
 * @property string displayName
 * @property string description
 * @property string confirm_uninstall
 */
class Imoje extends PaymentModule
{

	/**
	 * Sets the Information for the Module manager
	 * Also creates an instance of this class
	 */
	public function __construct()
	{
		$this->name = 'imoje';
		$this->tab = 'payments_gateways';
		$this->version = '1.15.0';
		$this->author = 'imoje';
		$this->need_instance = 1;
		$this->currencies = true;
		$this->currencies_mode = 'checkbox';
		$this->ps_versions_compliancy = [
			'min' => '1.6',
			'max' => '1.6',
		];

		parent::__construct();

		$this->displayName = $this->l('imoje');
		$this->description = $this->l('Visa, MasterCard, BLIK ect.');
		$this->confirm_uninstall = $this->l('Are you sure you want to uninstall imoje module?');
	}

	/**
	 * This function installs the imoje Module
	 *
	 * @return boolean
	 */
	public function install()
	{

		if(!(
			parent::install()
			&& $this->registerHook('payment')
			&& $this->registerHook('orderConfirmation')
			&& $this->registerHook('paymentReturn')
			&& $this->registerHook('displayBackOfficeHeader')
			&& $this->registerHook('displayHeader')
			&& $this->registerHook('header')
			&& Configuration::updateValue('IMOJE_ENABLED', 0)
			&& Configuration::updateValue('IMOJE_SANDBOX', 0)
			&& Configuration::updateValue('IMOJE_DEBUG_MODE', 0)
			&& Configuration::updateValue('IMOJE_CREATE_ORDER_ARRANGEMENT', 0)
			&& Configuration::updateValue('IMOJE_ING_KSIEGOWOSC', 0)
			&& Configuration::updateValue('IMOJE_ING_LEASE_NOW', 0)
			&& Configuration::updateValue('IMOJE_SERVICE_ID', '')
			&& Configuration::updateValue('IMOJE_SERVICE_KEY', '')
			&& Configuration::updateValue('IMOJE_MERCHANT_ID', '')
			&& Configuration::updateValue('IMOJE_AUTHORIZATION_TOKEN', '')

			&& Configuration::updateValue('IMOJE_GA_KEY', '')

			&& Configuration::updateValue('IMOJE_IMOJE_BUTTON', 1)
			&& Configuration::updateValue('IMOJE_PAYMENT_TITLE', '')
			&& Configuration::updateValue('IMOJE_CURRENCIES', [])
			&& Configuration::updateValue('IMOJE_HIDE_BRAND', 0)

			&& Configuration::updateValue('IMOJE_BLIK_BUTTON', 0)
			&& Configuration::updateValue('IMOJE_BLIK_PAYMENT_TITLE', '')
			&& Configuration::updateValue('IMOJE_BLIK_HIDE_BRAND', 0)
			&& Configuration::updateValue('IMOJE_BLIK_CODE_CHECKOUT', 0)
			&& Configuration::updateValue('IMOJE_BLIK_CURRENCIES', [])
			//&& Configuration::updateValue('IMOJE_BLIK_ONECLICK', 0)

			&& Configuration::updateValue('IMOJE_PBL_BUTTON', 0)
			&& Configuration::updateValue('IMOJE_PBL_PAYMENT_TITLE', '')
			&& Configuration::updateValue('IMOJE_PBL_HIDE_BRAND', 0)
			&& Configuration::updateValue('IMOJE_PBL_CURRENCIES', [])

			&& Configuration::updateValue('IMOJE_CARDS_BUTTON', 0)
			&& Configuration::updateValue('IMOJE_CARDS_PAYMENT_TITLE', '')
			&& Configuration::updateValue('IMOJE_CARDS_HIDE_BRAND', 0)
			&& Configuration::updateValue('IMOJE_CARDS_CURRENCIES', [])

			&& Configuration::updateValue('IMOJE_VISA_BUTTON', 0)
			&& Configuration::updateValue('IMOJE_VISA_PAYMENT_TITLE', '')
			&& Configuration::updateValue('IMOJE_VISA_HIDE_BRAND', 0)
			&& Configuration::updateValue('IMOJE_VISA_CURRENCIES', [])

			&& Configuration::updateValue('IMOJE_PAYLATER_BUTTON', 0)
			&& Configuration::updateValue('IMOJE_PAYLATER_PAYMENT_TITLE', '')
			&& Configuration::updateValue('IMOJE_PAYLATER_HIDE_BRAND', 0)
			&& Configuration::updateValue('IMOJE_PAYLATER_CURRENCIES', [])

		)) {

			return false;
		}

		if(Validate::isInt(Configuration::get('PAYMENT_IMOJE_NEW_STATUS'))
			xor (Validate::isLoadedObject($orderStateNew = new OrderState(Configuration::get('PAYMENT_IMOJE_NEW_STATUS'))))) {

			$orderStateNew = new OrderState();
			$missingLang = true;

			$langs = [
				'en' => 'Payment imoje: awaiting for confirmation',
				'pl' => 'Płatność imoje: oczekuje na potwierdzenie',
			];

			foreach($langs as $lang => $message) {
				$langId = Language::getIdByIso($lang);
				if(isset($langId)) {
					$orderStateNew->name[$langId] = $message;
					$missingLang = false;
				}
			}

			if($missingLang) {
				$langId = (int) $this->context->language->id;
				$orderStateNew->name[$langId] = $langs['en'];
			}

			$orderStateNew->send_email = false;
			$orderStateNew->invoice = false;
			$orderStateNew->unremovable = false;
			$orderStateNew->color = "lightblue";

			if(!$orderStateNew->add()) {
				$this->_errors[] = $this->l('There was an Error installing the module. Cannot add new order state.');

				return false;
			}
			if(!Configuration::updateValue('PAYMENT_IMOJE_NEW_STATUS', $orderStateNew->id)) {
				$this->_errors[] = $this->l('There was an Error installing the module. Cannot update new order state.');

				return false;
			}

			copy(_PS_MODULE_DIR_ . $this->name . '/logo.gif', _PS_IMG_DIR_ . 'os/' . $orderStateNew->id . '.gif');
		}

		return true;
	}

	/**
	 * This function uninstalls the imoje Module
	 *
	 * @return boolean
	 */
	public function uninstall()
	{

		return Configuration::deleteByName('IMOJE_ENABLED')
			&& Configuration::deleteByName('IMOJE_CREATE_ORDER_ARRANGEMENT')
			&& Configuration::deleteByName('IMOJE_SANDBOX')
			&& Configuration::deleteByName('IMOJE_DEBUG_MODE')
			&& Configuration::deleteByName('IMOJE_ING_KSIEGOWOSC')
			&& Configuration::deleteByName('IMOJE_ING_LEASE_NOW')

			&& Configuration::deleteByName('IMOJE_MERCHANT_ID')
			&& Configuration::deleteByName('IMOJE_SERVICE_ID')
			&& Configuration::deleteByName('IMOJE_SERVICE_KEY')
			&& Configuration::deleteByName('IMOJE_AUTHORIZATION_TOKEN')

			&& Configuration::deleteByName('IMOJE_IMOJE_BUTTON')
			&& Configuration::deleteByName('IMOJE_PAYMENT_TITLE')
			&& Configuration::deleteByName('IMOJE_HIDE_BRAND')
			&& Configuration::deleteByName('IMOJE_CURRENCIES')

			&& Configuration::deleteByName('IMOJE_BLIK_BUTTON')
			&& Configuration::deleteByName('IMOJE_BLIK_PAYMENT_TITLE')
			&& Configuration::deleteByName('IMOJE_BLIK_HIDE_BRAND')
			&& Configuration::deleteByName('IMOJE_BLIK_CODE_CHECKOUT')
			&& Configuration::deleteByName('IMOJE_BLIK_CURRENCIES')
			//&& Configuration::deleteByName('IMOJE_BLIK_ONECLICK')

			&& Configuration::deleteByName('IMOJE_CARDS_BUTTON')
			&& Configuration::deleteByName('IMOJE_CARDS_PAYMENT_TITLE')
			&& Configuration::deleteByName('IMOJE_CARDS_HIDE_BRAND')
			&& Configuration::deleteByName('IMOJE_CARDS_CURRENCIES')

			&& Configuration::deleteByName('IMOJE_VISA_BUTTON')
			&& Configuration::deleteByName('IMOJE_VISA_PAYMENT_TITLE')
			&& Configuration::deleteByName('IMOJE_VISA_HIDE_BRAND')
			&& Configuration::deleteByName('IMOJE_VISA_CURRENCIES')

			&& Configuration::deleteByName('IMOJE_PBL_BUTTON')
			&& Configuration::deleteByName('IMOJE_PBL_PAYMENT_TITLE')
			&& Configuration::deleteByName('IMOJE_PBL_HIDE_BRAND')
			&& Configuration::deleteByName('IMOJE_PBL_CURRENCIES')

			&& Configuration::deleteByName('IMOJE_PAYLATER_PAYMENT_TITLE')
			&& Configuration::deleteByName('IMOJE_PAYLATER_BUTTON')
			&& Configuration::deleteByName('IMOJE_PAYLATER_HIDE_BRAND')
			&& Configuration::deleteByName('IMOJE_PAYLATER_CURRENCIES')

			&& Configuration::deleteByName('IMOJE_GA_KEY')
			&& parent::uninstall();
	}

	/**
	 * Display configuration form.
	 *
	 * @return mixed
	 */
	public function getContent()
	{

		$msg = '';

		if(Tools::isSubmit('submitImoje')) {
			if($this->saveConfiguration()) {
				$msg = [
					'type'    => 'success',
					'message' => $this->l('Configuration updated successfully'),
				];
			} else {
				$msg = [
					'type'    => 'error',
					'message' => $this->l('There was an error saving your configuration'),
				];
			}
		}

		$this->context->smarty->assign([
			'imoje_form' => './index.php?tab=AdminModules&configure=imoje&token=' . Tools::getAdminTokenLite('AdminModules') . '&tab_module=' . $this->tab . '&module_name=imoje',

			'imoje_enabled' => Configuration::get('IMOJE_ENABLED'),

			'imoje_debug_mode' => Configuration::get('IMOJE_DEBUG_MODE'),

			'imoje_sandbox' => Configuration::get('IMOJE_SANDBOX'),

			'imoje_create_order_arrangement' => Configuration::get('IMOJE_CREATE_ORDER_ARRANGEMENT'),

			'imoje_ing_ksiegowosc' => Configuration::get('IMOJE_ING_KSIEGOWOSC'),

			'imoje_ing_lease_now' => Configuration::get('IMOJE_ING_LEASE_NOW'),

			'imoje_merchant_id'         => Configuration::get('IMOJE_MERCHANT_ID'),
			'imoje_service_id'          => Configuration::get('IMOJE_SERVICE_ID'),
			'imoje_service_key'         => Configuration::get('IMOJE_SERVICE_KEY'),
			'imoje_authorization_token' => Configuration::get('IMOJE_AUTHORIZATION_TOKEN'),

			'imoje_notification_url' => $this->context->link->getModuleLink('imoje', 'notification'),

			'imoje_msg' => $msg,

			'imoje_ga_key' => Configuration::get('IMOJE_GA_KEY'),

			'imoje_available_currencies' => \Imoje\Payment\Util::getSupportedCurrencies(),

			'imoje_imoje_button'          => Configuration::get('IMOJE_IMOJE_BUTTON'),
			'imoje_payment_title'         => Configuration::get('IMOJE_PAYMENT_TITLE'),
			'imoje_payment_title_default' => $this->getPaymentTitleDefault(),
			'imoje_hide_brand'            => Configuration::get('IMOJE_HIDE_BRAND'),
			'imoje_currencies'            => explode(",", Configuration::get('IMOJE_CURRENCIES')),

			'imoje_blik_button'                => Configuration::get('IMOJE_BLIK_BUTTON'),
			'imoje_blik_payment_title'         => Configuration::get('IMOJE_BLIK_PAYMENT_TITLE'),
			'imoje_blik_payment_title_default' => $this->getBlikPaymentTitleDefault(),
			'imoje_blik_hide_brand'            => Configuration::get('IMOJE_BLIK_HIDE_BRAND'),
			'imoje_blik_code_checkout'         => Configuration::get('IMOJE_BLIK_CODE_CHECKOUT'),
			'imoje_blik_currencies'            => explode(",", Configuration::get('IMOJE_BLIK_CURRENCIES')),
			//'imoje_blik_oneclick'              => Configuration::get('IMOJE_BLIK_ONECLICK'),
			'imoje_blik_oneclick'              => false,

			'imoje_cards_button'                => Configuration::get('IMOJE_CARDS_BUTTON'),
			'imoje_cards_payment_title'         => Configuration::get('IMOJE_CARDS_PAYMENT_TITLE'),
			'imoje_cards_payment_title_default' => $this->getCardsPaymentTitleDefault(),
			'imoje_cards_hide_brand'            => Configuration::get('IMOJE_CARDS_HIDE_BRAND'),
			'imoje_cards_currencies'            => explode(",", Configuration::get('IMOJE_CARDS_CURRENCIES')),

			'imoje_visa_button'                => Configuration::get('IMOJE_VISA_BUTTON'),
			'imoje_visa_payment_title'         => Configuration::get('IMOJE_VISA_PAYMENT_TITLE'),
			'imoje_visa_payment_title_default' => $this->getVisaPaymentTitleDefault(),
			'imoje_visa_hide_brand'            => Configuration::get('IMOJE_VISA_HIDE_BRAND'),
			'imoje_visa_currencies'            => explode(",", Configuration::get('IMOJE_VISA_CURRENCIES')),

			'imoje_paylater_button'                => Configuration::get('IMOJE_PAYLATER_BUTTON'),
			'imoje_paylater_payment_title'         => Configuration::get('IMOJE_PAYLATER_PAYMENT_TITLE'),
			'imoje_paylater_payment_title_default' => $this->getPaylaterPaymentTitleDefault(),
			'imoje_paylater_hide_brand'            => Configuration::get('IMOJE_PAYLATER_HIDE_BRAND'),
			'imoje_paylater_currencies'            => explode(",", Configuration::get('IMOJE_PAYLATER_CURRENCIES')),

			'imoje_pbl_button'                => Configuration::get('IMOJE_PBL_BUTTON'),
			'imoje_pbl_hide_brand'            => Configuration::get('IMOJE_PBL_HIDE_BRAND'),
			'imoje_pbl_payment_title'         => Configuration::get('IMOJE_PBL_PAYMENT_TITLE'),
			'imoje_pbl_payment_title_default' => $this->getPblPaymentTitleDefault(),
			'imoje_pbl_currencies'            => explode(",", Configuration::get('IMOJE_PBL_CURRENCIES')),
		]);

		return $this->fetchTemplate('admin.tpl');
	}

	/**
	 * @return bool
	 */
	private function saveConfiguration()
	{

		Configuration::updateValue('IMOJE_DEBUG_MODE', pSQL(Tools::getValue('imoje_debug_mode')));
		Configuration::updateValue('IMOJE_SANDBOX', pSQL(Tools::getValue('imoje_sandbox')));
		Configuration::updateValue('IMOJE_CREATE_ORDER_ARRANGEMENT', pSQL(Tools::getValue('imoje_create_order_arrangement')));
		Configuration::updateValue('IMOJE_ING_KSIEGOWOSC', pSQL(Tools::getValue('imoje_ing_ksiegowosc')));
		Configuration::updateValue('IMOJE_ING_LEASE_NOW', pSQL(Tools::getValue('imoje_ing_lease_now')));

		Configuration::updateValue('IMOJE_ENABLED', pSQL(Tools::getValue('imoje_enabled_form')));

		Configuration::updateValue('IMOJE_SERVICE_ID', pSQL(Tools::getValue('imoje_service_id')));
		Configuration::updateValue('IMOJE_SERVICE_KEY', pSQL(Tools::getValue('imoje_service_key')));
		Configuration::updateValue('IMOJE_MERCHANT_ID', pSQL(Tools::getValue('imoje_merchant_id')));
		Configuration::updateValue('IMOJE_AUTHORIZATION_TOKEN', pSQL(Tools::getValue('imoje_authorization_token')));

		Configuration::updateValue('IMOJE_GA_KEY', pSQL(Tools::getValue('imoje_ga_key')));

		Configuration::updateValue('IMOJE_IMOJE_BUTTON', pSQL(Tools::getValue('imoje_imoje_button')));
		Configuration::updateValue('IMOJE_PAYMENT_TITLE', pSQL(Tools::getValue('imoje_payment_title')));
		Configuration::updateValue('IMOJE_CURRENCIES', implode(',', Tools::getValue('imoje_currencies')));

		Configuration::updateValue('IMOJE_BLIK_BUTTON', pSQL(Tools::getValue('imoje_blik_button')));
		Configuration::updateValue('IMOJE_BLIK_PAYMENT_TITLE', pSQL(Tools::getValue('imoje_blik_payment_title')));
		Configuration::updateValue('IMOJE_BLIK_CODE_CHECKOUT', pSQL(Tools::getValue('imoje_blik_code_checkout')));
		//Configuration::updateValue('IMOJE_BLIK_ONECLICK', pSQL(Tools::getValue('imoje_blik_oneclick')));

		Configuration::updateValue('IMOJE_PBL_BUTTON', pSQL(Tools::getValue('imoje_pbl_button')));
		Configuration::updateValue('IMOJE_PBL_PAYMENT_TITLE', pSQL(Tools::getValue('imoje_pbl_payment_title')));

		Configuration::updateValue('IMOJE_PAYLATER_BUTTON', pSQL(Tools::getValue('imoje_paylater_button')));
		Configuration::updateValue('IMOJE_PAYLATER_PAYMENT_TITLE', pSQL(Tools::getValue('imoje_paylater_payment_title')));

		Configuration::updateValue('IMOJE_CARDS_BUTTON', pSQL(Tools::getValue('imoje_cards_button')));
		Configuration::updateValue('IMOJE_CARDS_PAYMENT_TITLE', pSQL(Tools::getValue('imoje_cards_payment_title')));

		Configuration::updateValue('IMOJE_VISA_BUTTON', pSQL(Tools::getValue('imoje_visa_button')));
		Configuration::updateValue('IMOJE_VISA_PAYMENT_TITLE', pSQL(Tools::getValue('imoje_visa_payment_title')));

		Configuration::updateValue('IMOJE_BLIK_HIDE_BRAND', pSQL(Tools::getValue('imoje_blik_hide_brand')));
		Configuration::updateValue('IMOJE_HIDE_BRAND', pSQL(Tools::getValue('imoje_hide_brand')));
		Configuration::updateValue('IMOJE_CARDS_HIDE_BRAND', pSQL(Tools::getValue('imoje_cards_hide_brand')));
		Configuration::updateValue('IMOJE_VISA_HIDE_BRAND', pSQL(Tools::getValue('imoje_visa_hide_brand')));
		Configuration::updateValue('IMOJE_PAYLATER_HIDE_BRAND', pSQL(Tools::getValue('imoje_paylater_hide_brand')));
		Configuration::updateValue('IMOJE_PBL_HIDE_BRAND', pSQL(Tools::getValue('imoje_pbl_hide_brand')));

		Configuration::updateValue('IMOJE_PAYLATER_ENABLED', pSQL(Tools::getValue('imoje_paylater')));

		if(is_array($imojeBlikCurrencies = Tools::getValue('imoje_blik_currencies'))) {
			Configuration::updateValue('IMOJE_BLIK_CURRENCIES', pSQL(implode(',', $imojeBlikCurrencies)));
		}

		if(is_array($imojeCurrencies = Tools::getValue('imoje_currencies'))) {
			Configuration::updateValue('IMOJE_CURRENCIES', pSQL(implode(',', $imojeCurrencies)));
		}

		if(is_array($imojePaylaterCurrencies = Tools::getValue('imoje_paylater_currencies'))) {
			Configuration::updateValue('IMOJE_PAYLATER_CURRENCIES', pSQL(implode(',', $imojePaylaterCurrencies)));
		}

		if(is_array($imojeCardsCurrencies = Tools::getValue('imoje_cards_currencies'))) {
			Configuration::updateValue('IMOJE_CARDS_CURRENCIES', pSQL(implode(',', $imojeCardsCurrencies)));
		}

		if(is_array($imojeVisaCurrencies = Tools::getValue('imoje_visa_currencies'))) {
			Configuration::updateValue('IMOJE_VISA_CURRENCIES', pSQL(implode(',', $imojeVisaCurrencies)));
		}

		if(is_array($imojePblCurrencies = Tools::getValue('imoje_pbl_currencies'))) {
			Configuration::updateValue('IMOJE_PBL_CURRENCIES', pSQL(implode(',', $imojePblCurrencies)));
		}

		return true;
	}

	/**
	 * @return string
	 */
	private function getPaymentTitleDefault()
	{
		return $this->l('Pay for purchases using the most convenient online payment methods - PayByLink, cards, BLIK');
	}

	/**
	 * @return string
	 */
	private function getBlikPaymentTitleDefault()
	{

		return $this->l('BLIK');
	}

	/**
	 * @return string
	 */
	private function getCardsPaymentTitleDefault()
	{

		return $this->l('Pay by cards');
	}

	/**
	 * @return string
	 */
	private function getVisaPaymentTitleDefault()
	{

		return $this->l('Visa Mobile payment with imoje');
	}

	/**
	 * @return string
	 */
	private function getPaylaterPaymentTitleDefault()
	{

		return $this->l('Twisto - imoje pay later');
	}

	/**
	 * @return string
	 */
	private function getPblPaymentTitleDefault()
	{

		return $this->l('Pay-By-Link with imoje');
	}

	/**
	 * @param string $name
	 *
	 * @return mixed
	 */
	public function fetchTemplate($name)
	{

		return $this->display(__FILE__, $name);
	}

	/**
	 * @param bool $http
	 * @param bool $entities
	 *
	 * @return string
	 */
	public function getModuleAddress($http = true, $entities = true)
	{
		return $this->getShopDomainAddress($http, $entities) . (__PS_BASE_URI__ . 'modules/' . $this->name . '/');
	}

	/**
	 * @param bool $http
	 * @param bool $entities
	 *
	 * @return string
	 */
	public static function getShopDomainAddress($http = false, $entities = false)
	{
		if(method_exists('Tools', 'getShopDomainSsl')) {
			return Tools::getShopDomainSsl($http, $entities);
		}

		if(!($domain = Configuration::get('PS_SHOP_DOMAIN_SSL'))) {
			$domain = Tools::getHttpHost();
		}

		if($entities) {
			$domain = htmlspecialchars($domain, ENT_COMPAT, 'UTF-8');
		}

		if($http) {
			$domain = (Configuration::get('PS_SSL_ENABLED')
					? 'https://'
					: 'http://') . $domain;
		}

		return $domain;
	}

	/**
	 * Hook that run on mobile version of Prestashop.
	 */
	public function hookDisplayMobileHeader()
	{
		$this->hookHeader();
	}

	/**
	 * @return mixed
	 */
	public function hookPaymentReturn()
	{
		return $this->fetchTemplate('success.tpl');
	}

	/**
	 * Hook that adds css and js in admin panel. For newer version of Prestashop.
	 */
	public function hookDisplayBackOfficeHeader()
	{

		$this->context->controller->addCSS($this->_path . 'assets/css/imoje-admin.css', 'all');

		if(!Configuration::get('IMOJE_AUTHORIZATION_TOKEN')
			|| !Configuration::get('IMOJE_MERCHANT_ID')
			|| !Configuration::get('IMOJE_SERVICE_ID')) {
			return '';
		}

		$vieworder = Tools::getValue('vieworder');
		$orderId = Tools::getValue('id_order');

		if($vieworder === false
			|| $orderId === false) {
			return '';
		}

		$order = new Order($orderId);
		$orderCurrentState = $order->getCurrentState();

		if($order->module !== 'imoje'
			|| ($orderCurrentState === Configuration::get('PAYMENT_IMOJE_NEW_STATUS')
				|| $orderCurrentState === _PS_OS_CANCELED_
				|| $orderCurrentState === _PS_OS_REFUND_)
		) {
			return '';
		}

		$orderPayment = $order->getOrderPaymentCollection()->getResults()[0];

		if(!isset($orderPayment->transaction_id)
			|| !$orderPayment->transaction_id
		) {
			return '';
		}

		$canRefund = true;
		$msg = '';
		$refundError = '';

		if(!Tools::getValue('submitImojePaymentRefund')) {

			$this->context->smarty->assign('imoje_can_refund', $canRefund);
			$this->context->smarty->assign('imoje_refund_message', $msg);
			$this->context->smarty->assign('imoje_refund_error', $refundError);

			return $this->fetchTemplate('/views/templates/admin/refunds.tpl');
		}

		if(empty($refundErrors)) {

			$api = new \Imoje\Payment\Api(
				Configuration::get('IMOJE_AUTHORIZATION_TOKEN'),
				Configuration::get('IMOJE_MERCHANT_ID'),
				Configuration::get('IMOJE_SERVICE_ID')
			);

			$currencyInfo = Currency::getCurrency($order->id_currency);

			$refund = $api->createRefund(
				$api->prepareRefundData(
					\Imoje\Payment\Util::convertAmountToFractional($order->total_paid)
				),
				$orderPayment->transaction_id
			);

			if(!$refund['success']) {
				if(isset($refund['data']['body']) && $refund['data']['body']) {
					$refundError = $this->l('Refund error:') . ' ' . $refund['data']['body'];
				} else {
					$refundError = $this->l('Refund error.');
				}

				PrestaShopLogger::addLog($refundError . ' ' . $this->l('order ID:') . ' ' . $orderId);
			} else {

				$isRefundSettled = true;

				if($refund['body']['transaction']['status'] !== 'settled') {

					$refundError = $this->l('Refund error.');
					$isRefundSettled = false;
				}

				if($isRefundSettled) {
					$msg = $this->l('Refund for amount:')
						. ' '
						. round($order->total_paid, 2)
						. ' '
						. $currencyInfo['iso_code']
						. ' '
						. $this->l('has been created.')
						. ' '
						. $this->l('order ID:')
						. ' '
						. $orderId;

					PrestaShopLogger::addLog($msg);

					$history = new OrderHistory();
					$history->id_order = $orderId;
					$history->changeIdOrderState(Configuration::get('PS_OS_REFUND'), $orderId);
					$history->addWithemail(true, []);

					$canRefund = false;
				}
			}
		}

		$this->context->smarty->assign('imoje_can_refund', $canRefund);
		$this->context->smarty->assign('imoje_refund_message', $msg);
		$this->context->smarty->assign('imoje_refund_error', $refundError);

		return $this->fetchTemplate('/views/templates/admin/refunds.tpl');
	}

	/**
	 * @param array $params
	 *
	 * @return bool|mixed
	 * @throws Exception
	 */
	public function hookPayment($params)
	{

		if(
			!Configuration::get('IMOJE_ENABLED')
			|| !Configuration::get('IMOJE_MERCHANT_ID')
			|| !Configuration::get('IMOJE_SERVICE_ID')
			|| !Configuration::get('IMOJE_SERVICE_KEY')
			|| !$this->active
		) {
			return false;
		}

		$currency = Currency::getCurrency($params['cart']->id_currency);

		$isCurrencyGlobal = \Imoje\Payment\Util::canUseForCurrency($currency['iso_code']);

		$this->smarty->assign([
			'imoje_ps_version' => _PS_VERSION_,

			'imoje_img'    => $this->getLogoPath(),
			'blik_img'     => $this->getLogoPath('blik-mini.png'),
			'paylater_img' => $this->getLogoPath('paylater.png'),
			'cards_img'    => $this->getLogoPath('cards.png'),
			'visa_img'     => $this->getLogoPath('visa.png'),
			'pbl_img'      => $this->getLogoPath('pbl.png'),

			'imoje_sandbox' => Configuration::get('IMOJE_SANDBOX'),

			'imoje_payment_url'          => $this->context->link->getModuleLink('imoje', 'payment'),
			'imoje_payment_blik_url'     => $this->context->link->getModuleLink('imoje', 'paymentblik'),
			'imoje_payment_cards_url'    => $this->context->link->getModuleLink('imoje', 'paymentcards'),
			'imoje_payment_pbl_url'      => $this->context->link->getModuleLink('imoje', 'pbl'),
			'imoje_payment_paylater_url' => $this->context->link->getModuleLink('imoje', 'paymentpaylater'),
			'imoje_payment_visa_url'     => $this->context->link->getModuleLink('imoje', 'paymentvisa'),

			'imoje_payment_title' => Configuration::get('IMOJE_PAYMENT_TITLE')
				?: $this->getPaymentTitleDefault(),
			'imoje_imoje_button'  => Configuration::get('IMOJE_IMOJE_BUTTON'),
			'imoje_hide_brand'    => Configuration::get('IMOJE_HIDE_BRAND'),
			'imoje_is_currency'   => $isCurrencyGlobal
				&& in_array($currency['iso_code'], explode(',', Configuration::get('IMOJE_CURRENCIES'))),

			'imoje_blik_button'        => Configuration::get('IMOJE_BLIK_BUTTON'),
			'imoje_blik_payment_title' => Configuration::get('IMOJE_BLIK_PAYMENT_TITLE')
				?: $this->getBlikPaymentTitleDefault(),
			'imoje_blik_hide_brand'    => Configuration::get('IMOJE_BLIK_HIDE_BRAND'),
			'imoje_blik_is_currency'   => $isCurrencyGlobal
				&& in_array(strtoupper($currency['iso_code']), explode(',', Configuration::get('IMOJE_BLIK_CURRENCIES'))),

			'imoje_pbl_button'        => Configuration::get('IMOJE_PBL_BUTTON'),
			'imoje_pbl_payment_title' => Configuration::get('IMOJE_PBL_PAYMENT_TITLE')
				?: $this->getPblPaymentTitleDefault(),
			'imoje_pbl_hide_brand'    => Configuration::get('IMOJE_PBL_HIDE_BRAND'),
			'imoje_pbl_is_currency'   => $isCurrencyGlobal
				&& in_array(strtoupper($currency['iso_code']), explode(',', Configuration::get('IMOJE_PBL_CURRENCIES'))),

			'imoje_cards_button'        => Configuration::get('IMOJE_CARDS_BUTTON'),
			'imoje_cards_payment_title' => Configuration::get('IMOJE_CARDS_PAYMENT_TITLE')
				?: $this->getCardsPaymentTitleDefault(),
			'imoje_cards_hide_brand'    => Configuration::get('IMOJE_CARDS_HIDE_BRAND'),
			'imoje_cards_is_currency'   => $isCurrencyGlobal
				&& in_array($currency['iso_code'], explode(',', Configuration::get('IMOJE_CARDS_CURRENCIES'))),

			'imoje_visa_button'        => Configuration::get('IMOJE_VISA_BUTTON'),
			'imoje_visa_payment_title' => Configuration::get('IMOJE_VISA_PAYMENT_TITLE')
				?: $this->getVisaPaymentTitleDefault(),
			'imoje_visa_hide_brand'    => Configuration::get('IMOJE_VISA_HIDE_BRAND'),
			'imoje_visa_is_currency'   => $isCurrencyGlobal
				&& in_array($currency['iso_code'], explode(',', Configuration::get('IMOJE_VISA_CURRENCIES'))),

			'imoje_paylater_enabled'       => Configuration::get('IMOJE_PAYLATER_ENABLED'),
			'imoje_paylater_button'        => Configuration::get('IMOJE_PAYLATER_BUTTON'),
			'imoje_paylater_payment_title' => Configuration::get('IMOJE_PAYLATER_PAYMENT_TITLE')
				?: $this->getPaylaterPaymentTitleDefault(),
			'imoje_paylater_hide_brand'    => Configuration::get('IMOJE_PAYLATER_HIDE_BRAND'),
			'imoje_paylater_is_currency'   => $isCurrencyGlobal
				&& in_array($currency['iso_code'], explode(',', Configuration::get('IMOJE_PAYLATER_CURRENCIES'))),

			'imoje_redirect_hint' => $this->l('Please wait, in the next step you will choose a payment method.'),
			'loading_gif'         => Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/assets/img/loading.gif'),
		]);

		return $this->fetchTemplate('payment.tpl');
	}

	/**
	 * @param string $file
	 *
	 * @return mixed
	 */
	public function getLogoPath($file = 'logo.svg')
	{
		return Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/assets/img/' . $file);
	}

	/**
	 * @param $currency_id
	 *
	 * @return mixed
	 */
	public function getCurrencyCode($currency_id)
	{
		return $this->getCurrencyById($currency_id)['iso_code'];
	}

	/**
	 * @param string $id
	 *
	 * @return string|null
	 */
	private function getCurrencyById($id)
	{
		$currency = null;
		$currencyList = $this->getCurrency($id);
		foreach($currencyList as $curr) {
			if($curr['id_currency'] === $id) {
				$currency = $curr;
				break;
			}
		}

		return $currency;
	}

	/**
	 * @param Cart   $cart
	 * @param string $visibleMethod
	 *
	 * @return array
	 */
	public function getDataForRequest($cart, $visibleMethod = '', $preselectMethodCode = '')
	{

		$cartId = $cart->id;

		$customer = new Customer(intval($cart->id_customer));

		$addressBilling = new Address((int) $cart->id_address_invoice);
		$addressDelivery = new Address((int) $cart->id_address_delivery);

		$currencyInfo = Currency::getCurrency($cart->id_currency);

		$total = \Imoje\Payment\Util::convertAmountToFractional($cart->getOrderTotal());

		$createOrderArrangement = Configuration::get('IMOJE_CREATE_ORDER_ARRANGEMENT');

		$successUrlIpn = $this->context->link->getPageLink('order-confirmation', true, null, [
			'id_cart'   => $cart->id,
			'key'       => $customer->secure_key,
			'id_module' => $this->id,
		]);
		$failureUrl = $this->context->link->getModuleLink('imoje', 'failure');

		$imojeVersion = $this->version . ';prestashop_' . _PS_VERSION_;

		$customerPhone = ($addressDelivery->phone_mobile
			?: ($addressBilling->phone_mobile
				?: ''));

		$invoice = Configuration::get('IMOJE_ING_KSIEGOWOSC')
			? \Imoje\Payment\Invoice::get($this->getCart($cart), $customer->email, false)
			: '';

		$leasenow = Configuration::get('IMOJE_ING_LEASE_NOW')
			? $this->getLeaseNow($cart)
			: '';

		// 0 - after checkout, 1 - after ipn
		if($createOrderArrangement) {

			return \Imoje\Payment\Paywall::prepareData(
				$total,
				$currencyInfo['iso_code'],
				$cartId,
				$customer->firstname,
				$customer->lastname,
				'',
				$customer->email,
				$customerPhone,
				$successUrlIpn,
				$failureUrl,
				$successUrlIpn,
				$imojeVersion,
				'',
				$visibleMethod,
				$invoice,
				$leasenow,
				$preselectMethodCode
			);
		}

		$this->validateOrder($cartId,
			Configuration::get('PAYMENT_IMOJE_NEW_STATUS'),
			$total,
			$this->displayName,
			null,
			[],
			null,
			false,
			$customer->secure_key
		);

		$orderId = Order::getOrderByCartId($cart->id);

		$successUrlCheckout = $this->context->link->getPageLink('order-confirmation', true, null, [
			'id_cart'   => $cart->id,
			'key'       => $customer->secure_key,
			'id_module' => $this->id,
			'id_order'  => $orderId,
		]);

		return \Imoje\Payment\Paywall::prepareData(
			$total,
			$currencyInfo['iso_code'],
			$orderId,
			$customer->firstname,
			$customer->lastname,
			'',
			$customer->email,
			$customerPhone,
			$successUrlCheckout,
			$failureUrl,
			$successUrlCheckout,
			$imojeVersion,
			'',
			$visibleMethod,
			$invoice,
			$leasenow,
			$preselectMethodCode
		);
	}

	/**
	 * @param object $cart
	 *
	 * @return string
	 */
	private function getLeaseNow($cart)
	{
		$leasenow = new \Imoje\Payment\LeaseNow();

		foreach($cart->getProducts() as $product) {

			$link = new Link();

			$category = new Category($product['id_category_default']);
			$categoryName = reset($category->name);

			$price = \Imoje\Payment\Util::convertAmountToFractional(round($product['price'], 2));
			$priceWt = \Imoje\Payment\Util::convertAmountToFractional(round($product['price_wt'], 2));

			$leasenow->addItem(
				$product['id_product'],
				$categoryName,
				$product['name'] . (empty($product['attributes'])
					? ''
					: ' - ' . $product['attributes']),
				$price,
				$priceWt - $price,
				$product['rate'],
				$product['quantity'],
				$link->getProductLink($product)
			);
		}

		return $leasenow->prepare();
	}

	/**
	 * @param Cart   $cart
	 * @param string $paymentMethod
	 * @param string $paymentMethodCode
	 * @param string $blikCode
	 *
	 * @return string
	 * @throws Exception
	 */
	public function getDataForRequestToApi($cart, $paymentMethod, $paymentMethodCode, $blikCode = '')
	{

		$customer = new Customer((int) $cart->id_customer);

		$currencyInfo = Currency::getCurrency($cart->id_currency);

		$cartId = (string) $cart->id;

		$isBlik = $paymentMethod === \Imoje\Payment\Util::getPaymentMethod('blik')
			&& (
				$paymentMethodCode === \Imoje\Payment\Util::getPaymentMethodCode('blik')
				|| $paymentMethodCode === \Imoje\Payment\Util::getPaymentMethod('blik_oneclick')
			);

		$imojeApi = new \Imoje\Payment\Api(
			Configuration::get('IMOJE_AUTHORIZATION_TOKEN'),
			Configuration::get('IMOJE_MERCHANT_ID'),
			Configuration::get('IMOJE_SERVICE_ID'),
			Configuration::get('IMOJE_SANDBOX')
				? \Imoje\Payment\Util::ENVIRONMENT_SANDBOX
				: \Imoje\Payment\Util::ENVIRONMENT_PRODUCTION
		);

		$total = $cart->getOrderTotal();
		$currency = $currencyInfo['iso_code'];
		$failureUrl = $this->context->link->getModuleLink('imoje', 'failure');
		$address = $this->getAddressData($cart);

		$cid = $this->context->customer->isLogged()
			? hash('md5', $this->context->customer->id . $this->context->customer->email)
			: '';

		$invoice = Configuration::get('IMOJE_ING_KSIEGOWOSC')
			? \Imoje\Payment\Invoice::get($this->getCart($cart), $customer->email, true)
			: '';

		// 0 - after checkout(order), 1 - after ipn(cart)
		if(Configuration::get('IMOJE_CREATE_ORDER_ARRANGEMENT')) {

			return $imojeApi->prepareData(
				$total,
				$currency,
				$cartId,
				$paymentMethod,
				$paymentMethodCode,
				Configuration::get('IMOJE_GA_KEY')
					? $this->context->link->getModuleLink('imoje', 'success', [
					'ga_cart_id' => $cartId,
					'ga_hash'    => hash('sha256', $cartId . $cart->secure_key),
				])
					: $this->context->link->getModuleLink('imoje', 'success'),
				$failureUrl,
				$customer->firstname,
				$customer->lastname,
				$customer->email,
				\Imoje\Payment\Api::TRANSACTION_TYPE_SALE,
				'',
				($blikCode && $isBlik)
					? $blikCode
					: '',
				$address,
				$cid,
				$invoice
			);
		}

		$imoje = new Imoje();

		$imoje->validateOrder($cartId,
			Configuration::get('PAYMENT_IMOJE_NEW_STATUS'),
			$total,
			$imoje->displayName,
			null,
			[],
			null,
			false,
			$customer->secure_key
		);

		$orderId = (string) Order::getOrderByCartId((int) $cartId);

		return $imojeApi->prepareData(
			$total,
			$currency,
			$orderId,
			$paymentMethod,
			$paymentMethodCode,
			_PS_BASE_URL_ . __PS_BASE_URI__ . 'index.php?controller=order-confirmation&id_cart=' . $cart->id . '&id_module=' . $imoje->id . '&id_order=' . $orderId . '&key=' . $customer->secure_key,
			$failureUrl,
			$customer->firstname,
			$customer->lastname,
			$customer->email,
			\Imoje\Payment\Api::TRANSACTION_TYPE_SALE,
			'',
			($blikCode && $isBlik)
				? $blikCode
				: '',
			$address,
			$cid,
			$invoice
		);
	}

	/**
	 * @param object $cart
	 *
	 * @return array
	 */
	private static function getAddressData($cart)
	{

		$addressBilling = new Address((int) $cart->id_address_invoice);
		$addressDelivery = new Address((int) $cart->id_address_delivery);

		return [
			'billing'  => \Imoje\Payment\Api::prepareAddressData(
				$addressBilling->firstname,
				$addressBilling->lastname,
				empty($addressBilling->address2)
					? $addressBilling->address1
					: $addressBilling->address1 . ', ' . $addressBilling->address2,
				$addressBilling->city,
				$addressBilling->state,
				$addressBilling->postcode,
				CountryCore::getIsoById($addressBilling->id_country)
			),
			'shipping' => \Imoje\Payment\Api::prepareAddressData(
				$addressDelivery->firstname,
				$addressDelivery->lastname,
				empty($addressDelivery->address2)
					? $addressDelivery->address1
					: $addressDelivery->address1 . ', ' . $addressDelivery->address2,
				$addressDelivery->city,
				$addressDelivery->state,
				$addressDelivery->postcode,
				CountryCore::getIsoById($addressDelivery->id_country)
			),
		];
	}

	/**
	 * @param Cart $cart
	 *
	 * @return \Imoje\Payment\CartData
	 */
	public function getCart($cart)
	{

		$cartData = new \Imoje\Payment\CartData();

		foreach($cart->getProducts() as $product) {

			$rate = (float) $product['rate'];

			if(isset($product['tax_name']) && $product['tax_name'] && strtolower($product['tax_name']) === \Imoje\Payment\Invoice::SHOP_TAX_EXEMPT) {
				$rate = explode('_', \Imoje\Payment\Invoice::TAX_EXEMPT)[1];
			}


			$cartData->addItem(
				$product['id_product'] . (empty($product['attributes_small'])
					? ''
					: ' - ' . $product['attributes_small']),
				$rate,
				$product['name'],
				\Imoje\Payment\Util::convertAmountToFractional(round($product['total_wt'], 2)),
				(float) $product['quantity'],
				false
			);
		}

		$cartData->setAmount(\Imoje\Payment\Util::convertAmountToFractional($cart->getOrderTotal()));
		$cartData->setCreatedAt(strtotime($cart->date_add));

		$carrier = new Carrier($cart->id_carrier);

		$addressBilling = new Address((int) $cart->id_address_invoice);
		$cartData->setAddressBilling(
			$addressBilling->city,
			$addressBilling->firstname . ' ' . $addressBilling->lastname,
			empty($addressBilling->phone_mobile)
				? (empty($addressBilling->phone)
				? ''
				: $addressBilling->phone)
				: $addressBilling->phone_mobile,
			empty($addressBilling->address2)
				? $addressBilling->address1
				: $addressBilling->address1 . ', ' . $addressBilling->address2,
			CountryCore::getIsoById($addressBilling->id_country),
			$addressBilling->postcode,
			empty($addressBilling->vat_number)
				? ''
				: $addressBilling->vat_number);

		$addressDelivery = new Address((int) $cart->id_address_delivery);
		$cartData->setAddressDelivery(
			$addressDelivery->city,
			$addressDelivery->firstname . ' ' . $addressDelivery->lastname,
			empty($addressDelivery->phone_mobile)
				? (empty($addressDelivery->phone)
				? ''
				: $addressDelivery->phone)
				: $addressDelivery->phone_mobile,
			empty($addressDelivery->address2)
				? $addressDelivery->address1
				: $addressDelivery->address1 . ', ' . $addressDelivery->address2,
			CountryCore::getIsoById($addressDelivery->id_country),
			$addressDelivery->postcode,
			empty($addressDelivery->vat_number)
				? ''
				: $addressDelivery->vat_number);

		$cartData->setShipping(
			$carrier->getTaxesRate(new Address((int) $cart->{Configuration::get('PS_TAX_ADDRESS_TYPE')})),
			$carrier->name,
			\Imoje\Payment\Util::convertAmountToFractional($cart->getTotalShippingCost())
		);

		$discount = $cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS);
		if($discount > 0) {
			$cartData->setDiscount(
				0,
				'discount',
				\Imoje\Payment\Util::convertAmountToFractional($discount)
			);
		}

		return $cartData;
	}
}
